package be.ac.ulb.code.wit.deepthought.jms;

import javax.jms.Connection;
import javax.jms.DeliveryMode;
import javax.jms.Destination;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;

import org.apache.activemq.ActiveMQConnection;
import org.apache.activemq.ActiveMQConnectionFactory;

public class JMSServiceClient {

    private static ActiveMQConnectionFactory connectionFactory;
    private static Connection connection;
    private static Session session;
    private static Destination destination;
    
    private static void setUp() throws JMSException {
        connectionFactory = new ActiveMQConnectionFactory(
                ActiveMQConnection.DEFAULT_USER,
                ActiveMQConnection.DEFAULT_PASSWORD,
                ActiveMQConnection.DEFAULT_BROKER_URL);
        connection = connectionFactory.createConnection();
        connection.start();
        session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        destination = session.createTopic("My topic");
    }
    
    private static void tearDown() throws JMSException {
        session.close();
        connection.stop();
        connection.close();
    }

    public static void main(String[] args) throws Exception {
        //Attach to the queue on which we can send messages
        setUp();
        
        //Read messages
        ResponseConsumer response = submitQuestion();
        
        //Do something while waiting for the answer
        int i = 0;
        while (!response.gotTheAnswer) {
        	System.out.println("Doing something important! (" + i + ")");
        	Thread.sleep(1000);
        	i++;
        }
        
        tearDown();
    }
    
    private static ResponseConsumer submitQuestion() throws JMSException {
    	//Make sure we listen for the answer
        MessageConsumer consumer = session.createConsumer(destination);
        ResponseConsumer myConsumer = new ResponseConsumer();
        connection.setExceptionListener(myConsumer);
        consumer.setMessageListener(myConsumer);
        
        //Send the message
        MessageProducer producer;
		producer = session.createProducer(destination);
        producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
        
        TextMessage message = session.createTextMessage("computeAnswer");
        producer.send(message);
        producer.close();
        
        return myConsumer;
    }

    private static class ResponseConsumer implements MessageListener, ExceptionListener {
    	public boolean gotTheAnswer = false;
    	
        synchronized public void onException(JMSException ex) { }

        public void onMessage(Message message) {
        	try {
        		if (message instanceof TextMessage) {
        			TextMessage textMessage = (TextMessage) message;

        			if (!textMessage.getText().equals("computeAnswer")) {
	    				message.acknowledge();
	    				System.out.println("The message is saying: " + textMessage.getText());
	    				gotTheAnswer = true;
        			}
        		}
        	} catch (JMSException e) {
        		e.printStackTrace();
        	}
        }
    }
}