package be.ac.ulb.code.wit.deepthought.jms;

import javax.jms.JMSException;

import org.apache.activemq.ActiveMQConnectionFactory;
import org.apache.activemq.broker.BrokerService;
import javax.jms.Connection;
import javax.jms.DeliveryMode;
import javax.jms.Destination;
import javax.jms.ExceptionListener;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import org.apache.activemq.ActiveMQConnection;

import be.ac.ulb.code.wit.deepthought.server.DeepThoughtImpl;

public class JMSServiceServer {

    private static ActiveMQConnectionFactory connectionFactory;
    private static Connection connection;
    private static Session session;
    private static Destination destination;
    
    private static void setUp() throws JMSException {
        connectionFactory = new ActiveMQConnectionFactory(
                ActiveMQConnection.DEFAULT_USER,
                ActiveMQConnection.DEFAULT_PASSWORD,
                ActiveMQConnection.DEFAULT_BROKER_URL);
        connection = connectionFactory.createConnection();
        connection.start();
        session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        destination = session.createTopic("My topic");
    }

    public static void main(String[] args) throws Exception {
    	//Instantiate a broker. In practice, this would be instantiated outside the application
        BrokerService broker = new BrokerService();
        broker.setUseJmx(true);
        broker.addConnector("tcp://localhost:61616");
        broker.start();

        //Create a queue on which to read incoming messages
        setUp();
        
        //Read messages
        process();

        System.out.println("Waiting for new messages...");
		System.in.read();
        
        broker.stop();
    }

    private static void process() throws JMSException, InterruptedException {
    	//Create a new ServiceConsumer object, and make it handle
    	// incoming messages/faults
        MessageConsumer consumer = session.createConsumer(destination);
        ServiceConsumer myConsumer = new ServiceConsumer();
        connection.setExceptionListener(myConsumer);
        consumer.setMessageListener(myConsumer);
    }

    private static class ServiceConsumer implements MessageListener, ExceptionListener {
    	//The actual implementation
    	DeepThoughtImpl impl = new DeepThoughtImpl();
        MessageProducer producer;
        
    	public ServiceConsumer() throws JMSException {
    		producer = session.createProducer(destination);
	        producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
        }
    	
        synchronized public void onException(JMSException ex) { }

        public void onMessage(Message message) {
        	try {
        		//Only handle text messages that say "computeAnswer"
        		if (message instanceof TextMessage) {
        			TextMessage textMessage = (TextMessage) message;

        			if (textMessage.getText().equals("computeAnswer")) {
        				message.acknowledge();
        				System.out.println("Computing the answer...");

        				//Compute the response
        				String response = impl.computeAnswer();

        				//Send the response
        				TextMessage responseMessage = session.createTextMessage(response);
        				producer.send(responseMessage);
        			}
        		}
        	} catch (JMSException e) {
        		e.printStackTrace();
        	}
        }
    }
}