package be.ac.ulb.code.wit.deepthought.server;

import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import javax.xml.namespace.QName;
import javax.xml.ws.Service;
import javax.xml.ws.wsaddressing.W3CEndpointReference;

import org.apache.cxf.helpers.DOMUtils;
import org.apache.cxf.ws.addressing.EndpointReferenceType;
import org.apache.cxf.wsdl.EndpointReferenceUtils;
import org.w3c.dom.Element;

import be.ac.ulb.code.wit.deepthought.Callback;

public class CallbackThread extends Thread {
	//See 
	//http://omgo.wordpress.com/2010/01/06/ws-addresssing-with-cxf-for-async-calls/
	//for details
	
    class PendingTask {
        EndpointReferenceType epr;
        String response;

        public PendingTask(EndpointReferenceType epr, String response) {
            super();
            this.epr = epr;
            this.response = response;
        }
    }

    boolean run = true;
    BlockingQueue<PendingTask> queue = new LinkedBlockingQueue<PendingTask>(5);

    //Add a new service to be called later
    public void scheduleCallback(EndpointReferenceType eprType, String message) {
        try {
            queue.put(new PendingTask(eprType, message));
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }
    
    @Override
    public void run() {
        System.out.println("Callback Thread started.. ");
        
        while (run) {
            try {
                PendingTask task = queue.take();
                if (task.epr != null) {
                    process(task);
                }
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
	}

    public CallbackThread() {
		this.start();
    }

    private void process(PendingTask task) throws InterruptedException {
        Service service = createService(task.epr);
        if (service != null) {
            W3CEndpointReference reference = 
            		new W3CEndpointReference(
            			EndpointReferenceUtils.convertToXML(task.epr));
            Callback callback = service.getPort(reference, Callback.class);
            callback.callBack(task.response);
        }
    }

    private static boolean isAddressingMetadata(Element element) {
    	return "ServiceName".equals(element.getLocalName()) && 
            "http://www.w3.org/2007/05/addressing/metadata".equals(
            		element.getNamespaceURI());
    }
    
    private static String getNamespace(Element metadata) {
        String serviceQnameString = metadata.getTextContent().trim();
        return DOMUtils.getNamespace(metadata, 
        		serviceQnameString.substring(0, serviceQnameString.indexOf(":")));
    }
    
    private static QName getServiceQName(Element metadata) {
        String serviceName = metadata.getTextContent().trim();
        serviceName = serviceName.substring(serviceName.indexOf(":") + 1);
        
        return new QName(getNamespace(metadata), serviceName);
    }
    
    private static QName getPortQName(Element metadata) {
        String endpointName = metadata.getAttribute("EndpointName");
        return new QName(getNamespace(metadata), endpointName);
    }
    
    private static Service createService(EndpointReferenceType referenceType) {
        List<Object> any = referenceType.getMetadata().getAny();
        QName serviceName = null;
        QName endpointName = null;

        for (Object a : any) {
            Element element = (Element) a;
            	
            if (isAddressingMetadata(element)) {
            	serviceName = getServiceQName(element);
            	endpointName = getPortQName(element);
            }
        }

        if (serviceName != null && endpointName != null) {
			//TODO: cache
            Service service = Service.create(serviceName);
            service.getPort(Callback.class);
            		//SOAPBinding.SOAP12HTTP_BINDING, referenceType.getAddress().getValue());
            return service;
        }

        return null;
    }

    public void stopProcessing() {
    	run = false;
        queue.add(new PendingTask(null, null)); //Ensure that we do no keep waiting on an empty queue
    }
}