package be.ac.ulb.code.wit.deepthought.soap.client;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.Endpoint;
import javax.xml.ws.Service;

import org.apache.cxf.BusFactory;
import org.apache.cxf.ws.addressing.AddressingPropertiesImpl;
import org.apache.cxf.ws.addressing.JAXWSAConstants;
import org.apache.cxf.ws.addressing.VersionTransformer;

import be.ac.ulb.code.wit.deepthought.Callback;
import be.ac.ulb.code.wit.deepthought.DeepThought;
import be.ac.ulb.code.wit.deepthought.DeepThoughtConsts;

public class WSAServiceClient {
	
	private static DeepThought getService() throws MalformedURLException {
		Service service = Service.create(
				new URL(DeepThoughtConsts.SERVICEADDRESS + "?wsdl"),
				DeepThoughtConsts.SERVICE);
		
		return service.getPort(DeepThought.class);
	}

    private static Endpoint startCallBackEndpoint(Callback implementor) {
        return Endpoint.publish(DeepThoughtConsts.CALLBACKADDRESS, implementor);
    }
	
    private static void setReplyDestination(DeepThought service, Endpoint replyTo) {
        AddressingPropertiesImpl maps = new AddressingPropertiesImpl();
        maps.setReplyTo(
        		VersionTransformer.convertToInternal(
				replyTo.getEndpointReference()));

        Map<String, Object> requestContext = 
        		((BindingProvider)service).getRequestContext();
        requestContext.put(JAXWSAConstants.CLIENT_ADDRESSING_PROPERTIES, maps);
    }
    
	private static Endpoint computeAnswer(DeepThought deepthought, Callback impl) {
		System.out.println("Computing the answer");
		
    	Endpoint callbackEP = startCallBackEndpoint(impl);
    	setReplyDestination(deepthought, callbackEP);
    	
    	deepthought.computeAnswerOneWay();
    	
        return callbackEP;
	}

	
	public static void main(String args[]) throws MalformedURLException {
		
		//Connect to the service
		DeepThought d = getService();
		
		//Create a new service that will be called back
    	CallbackImpl callback = new CallbackImpl();
		Endpoint callbackEP = computeAnswer(d, callback);
		
		int i = 0;
		while (!callback.isDone()) {
			System.out.println("Doing some work... (" + i + ")");
			++i;
			try { Thread.sleep(1000);
			} catch (InterruptedException e) { }
		} 
		
		System.out.println("Shutting down...");
        callbackEP.stop();
    	BusFactory.getDefaultBus().shutdown(true);
	}
}
