package be.ac.ulb.code.wit.deepthought.soap.server;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.apache.cxf.Bus;
import org.apache.cxf.binding.soap.SoapMessage;
import org.apache.cxf.binding.soap.SoapVersion;
import org.apache.cxf.binding.soap.SoapVersionFactory;
import org.apache.cxf.binding.soap.interceptor.AbstractSoapInterceptor;
import org.apache.cxf.bus.CXFBusFactory;
import org.apache.cxf.endpoint.Server;
import org.apache.cxf.endpoint.ServerRegistry;
import org.apache.cxf.helpers.IOUtils;
import org.apache.cxf.interceptor.InterceptorChain;
import org.apache.cxf.interceptor.StaxInInterceptor;
import org.apache.cxf.io.CachedOutputStream;
import org.apache.cxf.message.Message;
import org.apache.cxf.phase.Phase;
import org.apache.cxf.staxutils.DepthXMLStreamReader;
import org.apache.cxf.staxutils.StaxUtils;
import org.apache.cxf.transport.MessageObserver;

import be.ac.ulb.code.wit.deepthought.DeepThoughtConsts;

public class RouterInterceptor extends AbstractSoapInterceptor {
	Map<String, Server> targetServers = new HashMap<String, Server>();
	Server defaultTarget;

	private void enumerateServices() {
		//Look up for all available endpoints registered on the bus
		Bus bus = CXFBusFactory.getDefaultBus();
		ServerRegistry serverRegistry = bus.getExtension(ServerRegistry.class);
		List<Server> servers = serverRegistry.getServers();
		Double maxver = 0.0;

		for (Server server : servers) {
			String address = server.getEndpoint().getEndpointInfo().getAddress();
			String version = address.substring(address.lastIndexOf("/") + 1);

			try {
				Double ver = Double.parseDouble(version);

				if (ver > maxver) {
					defaultTarget = server;
					ver = maxver;
				}

				targetServers.put(version, server);
			}
			catch (NumberFormatException nfe) {}
		}
	}

	public RouterInterceptor() {
		super(Phase.POST_STREAM);
		addBefore(StaxInInterceptor.class.getName());
		enumerateServices();
	}

	private void redirect(SoapMessage message, Server target) {
		//Redirect the request
		MessageObserver mo = target.getDestination().getMessageObserver();
		mo.onMessage(message);

		//Now the response has been put in the message, abort the chain 
		InterceptorChain chain = message.getInterceptorChain();
		chain.abort();
	}

	private String getServiceVersion(InputStream is, String encoding) throws IOException, XMLStreamException, FactoryConfigurationError {
		XMLStreamReader xmlReader = XMLInputFactory.newFactory().
				createXMLStreamReader(is, encoding);
		DepthXMLStreamReader depthReader = new DepthXMLStreamReader(xmlReader);

		if (depthReader.nextTag() == XMLStreamConstants.START_ELEMENT) {
			SoapVersion soapVersion = 
					SoapVersionFactory.getInstance().getSoapVersion(xmlReader.getNamespaceURI());
			
			if (!xmlReader.getLocalName().equals(soapVersion.getEnvelope().getLocalPart())) {
				return null;
			}
			
			if (StaxUtils.toNextTag(depthReader, soapVersion.getHeader())) {
				if (StaxUtils.toNextTag(depthReader, DeepThoughtConsts.SERVICE_VERSION)) {
					StringBuilder sb = new StringBuilder();
					while (depthReader.next() == XMLStreamConstants.CHARACTERS) {
						sb.append(depthReader.getText());
					}

					if (depthReader.getEventType() == XMLStreamConstants.END_ELEMENT) {
						return sb.toString();
					}
					else {
						throw new RuntimeException("Invalid version element");
					}
				}
			}
		}

		return null;
	}

	private InputStream bufferMessageStream(SoapMessage message) throws IOException {
		InputStream is = message.getContent(InputStream.class);
		if (is != null) {
			CachedOutputStream bos = new CachedOutputStream();
			IOUtils.copy(is, bos);
			bos.flush();
			is.close();
			message.setContent(InputStream.class, bos.getInputStream());	

			IOUtils.copy(bos.getInputStream(), System.out);
			return bos.getInputStream();
		}
		return null;
	}

	public void handleMessage(SoapMessage message) {
		Server target = defaultTarget;

		try {
			InputStream is = bufferMessageStream(message);
			String version = getServiceVersion(is, (String)message.get(Message.ENCODING));

			if (version != null && targetServers.containsKey(version)) {
				target = targetServers.get(version);
			}
		} catch (XMLStreamException e1) {
			//Try the default target instead
			target=defaultTarget;
		} catch (Exception e1) {
			e1.printStackTrace();
			target=defaultTarget;
		}

		redirect(message, target);
	}
}